<?php

namespace Drupal\multi_crop_image_widget_crop\Plugin\MultiCrop;

use Drupal\multi_crop\MultiCropPluginBase;

/**
 * Plugin implementation of the multi_crop.
 *
 * @MultiCrop(
 *   id = "image_widget_crop",
 *   target_field_name = "image_crop",
 *   label = @Translation("Plugin image_widget_crop"),
 *   description = @Translation("Manage Multicrop for image_widget_crop.")
 * )
 */
class ImageWidgetCrop extends MultiCropPluginBase {

  /**
   * {@inheritdoc}
   */
  public function saveCrop(mixed $crop_settings, string $image_style, string $old_uri, string $new_uri, int $width, int $height) {

    // Filter to the concerned crop setting.
    foreach ($crop_settings['crop_wrapper'] as $crop_type => $container) {
      if ($crop_type != $image_style) {
        continue;
      }

      // Get Crop value.
      $crop_values = $container['crop_container']['values'];

      // If crop are set.
      if ($crop_values['crop_applied'] == '1') {

        // Recover Crop.
        $crop = $this->retrievCrop($crop_type, $old_uri, $new_uri);

        // Get Center coordinate of crop zone on original image.
        $axis_coordinate = \Drupal::service('image_widget_crop.manager')->getAxisCoordinates(
          ['x' => $crop_values['x'], 'y' => $crop_values['y']],
          ['width' => $crop_values['width'], 'height' => $crop_values['height']]
        );

        // Set position & size.
        $crop->setPosition($axis_coordinate['x'], $axis_coordinate['y']);
        $crop->setSize($crop_values['width'], $crop_values['height']);

        // Save Crop.
        $crop->save();

      }
    }
  }

}
