<?php

namespace Drupal\multi_crop\Plugin\Field\FieldFormatter;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;
use Drupal\media\MediaInterface;

/**
 * Plugin implementation of the 'image' formatter.
 *
 * @FieldFormatter(
 *   id = "image_multicrop",
 *   label = @Translation("Multi Crop Image"),
 *   field_types = {
 *     "image"
 *   },
 * )
 */
class MulticropFormatter extends ImageFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    $data = [];
    $entity_parent = $items->getEntity() ?? NULL;

    // If parent is a media.
    if ($entity_parent != NULL && $entity_parent instanceof MediaInterface) {
      $crop_context_ref = $entity_parent->entity_reference_entity_modify ?? NULL;
      $media_override = $entity_parent->_referringItem->getValue() ?? [];
      // If there are overwitten data.
      if (isset($media_override['overwritten_property_map'])) {
        $data = Json::decode($media_override['overwritten_property_map']);
      }
    }

    if ($data != []) {
      foreach ($data as $field_name => $data_values) {
        // If there are overwritten data.
        if ($items->getName() == $field_name) {
          foreach ($elements as $key => $item) {

            if (isset($data_values[$key])) {

              $crop_settings = $this->getPluginSetting($data_values[$key]);
              // Send crop data to custom image formater.
              $elements[$key]['#multi_crop_attributes'] = $crop_settings;
              $elements[$key]['#multi_crop_attributes']['context'] = $crop_context_ref;
              $elements[$key]['#multi_crop_attributes']['image_style'] = $elements[$key]['#image_style'];
              $elements[$key]['#theme'] = 'multi_crop_image_formatter';
            }
          }
        }
      }
    }

    return $elements;
  }

  /**
   * Recover Crop settings.
   *
   * @param array $configuration
   *   Field information.
   *
   * @return array
   *   Crop settings.
   */
  private function getPluginSetting(array $configuration) {

    $multi_crop_plugins = \Drupal::service('plugin.manager.multi_crop');
    $plugin_definitions = $multi_crop_plugins->getDefinitions();

    foreach ($plugin_definitions as $plugin_id => $plugin) {
      foreach ($configuration as $key => $value) {

        $target_field = $plugin['target_field_name'];
        if ($key === $target_field) {
          return [
            'plugin_id' => $plugin_id,
            'crop_setting' => $value,
          ];
        }
      }
    }
  }

}
